/**
 * @file main.h
 * @author Nippon Pulse Motor Co., Ltd.
 * @brief メインソースヘッダー
 * @version 0.1
 * @date 2023-06-07
 * 
 * @copyright Copyright (c) 2023
 * 
 */

#ifndef __MAIN_H__
#define __MAIN_H__

//----------------------------------------------------------------------------------------
// マクロ定義
//----------------------------------------------------------------------------------------
#define PIO_OUT_BASE        ( 0 )           /* PCLアドレスバスA0に接続するGPIOの番号を指定 */
#define PIO_OUT_NUM         ( 18 )          /* PCLアドレスバスとデータバスに使用する本数を指定 */
#define PIO_SIDE_BASE       ( 18 )          /* CS,WR,RDを割り当てるピンのベース番号を指定(PCLアドレスバスとデータバスの上にCS,RD,WRが来る)*/
#define PIO_SIDE_BASE_NUM   ( 3 )           /* CS,WR,RDに使用するピンの本数を指定(3本なので[3]) */
#define PIO_IN_BASE         ( 10 )          /* PCLデータバスD0に接続するGPIOの番号を指定 */
#define IFB                 ( 28 )          /* PCL_IFB信号読み込み端子を指定する */

//---------------------------------------------------------------------------------------
// 定数定義
//---------------------------------------------------------------------------------------
/**
 * @brief 軸選択
 * 
 */
enum Axis{
  X_SEL = 0x00,                          /* X軸選択 */
  Y_SEL = 0x01,                          /* Y軸選択 */
  Z_SEL = 0x02,                          /* Z軸選択 */
  U_SEL = 0x03,                          /* U軸選択 */
};

/**
 * @brief PCL6046レジスタアドレス(Z80,フルアドレス)
 * 
 */
enum Register{
  MSTS = 0x0001,                         /* メインステータス */
  COMB = 0x0100,                         /* コマンド */
  IOPB = 0x02,                           /* 汎用入出力ポート */
  OTPB = 0x02,                           /* 汎用出力ポート */
  SSTSB = 0x03,                          /* サブステータス */
  PRMV = 0x08090A0B,                     /* 移動量 プリレジスタ */
  PRFL = 0x0C0D0E0F,                     /* FL速度ステップ数 プリレジスタ */
  PRFH = 0x10111213,                     /* FH速度ステップ数 プリレジスタ */
  PRUR = 0x14151617,                     /* 加速レート プリレジスタ */
  PRDR = 0x18191A1B,                     /* 減速レート プリレジスタ */
  PRMG = 0x1C1D1E1F,                     /* 速度倍率 プリレジスタ */
  PRDP = 0x20212223,                     /* スローダウンポイント プリレジスタ */
  PRMD = 0x24252627,                     /* 動作モード プリレジスタ */
  PRIP = 0x28292A2B,                     /* 円弧補間中心位置 プリレジスタ */
  PRUS = 0x2C2D2E2F,                     /* 加速S字区間 プリレジスタ */
  PRDS = 0x30313233,                     /* 減速S字区間 プリレジスタ */
  PRCP5 = 0x34353637,                    /* コンパレータ5比較値 プリレジスタ */
  PRCI = 0x38393A3B,                     /* 円弧補間歩進数 プリレジスタ */
  RMV = 0x40414243,                      /* 移動量 レジスタ */
  RFL = 0x44454647,                      /* FL速度ステップ数 レジスタ */
  RFH = 0x48494A4B,                      /* FH速度ステップ数 レジスタ */
  RUR = 0x4C4D4E4F,                      /* 加速レート レジスタ */
  RDR = 0x50515253,                      /* 減速レート レジスタ */
  RMG = 0x54555657,                      /* 速度倍率 レジスタ */
  RDP = 0x58595A5B,                      /* スローダウンポイント レジスタ */
  RMD = 0x5C5D5E5F,                      /* 動作モード レジスタ */
  RIP = 0x60616263,                      /* 円弧補間中心位置 レジスタ */
  RUS = 0x64656667,                      /* 加速S字区間 レジスタ */
  RDS = 0x68696A6B,                      /* 減速S字区間 レジスタ */
  RFA = 0x6C6D6E6F,                      /* FA速度ステップ数 レジスタ */
  RENV1 = 0x70717273,                    /* 環境設定1 レジスタ */
  RENV2 = 0x74757677,                    /* 環境設定2 レジスタ */
  RENV3 = 0x78897A7B,                    /* 環境設定3 レジスタ */
  RENV4 = 0x7C7D7E7F,                    /* 環境設定4 レジスタ */
  RENV5 = 0x80818283,                    /* 環境設定5 レジスタ */
  RENV6 = 0x84858687,                    /* 環境設定6 レジスタ */
  RENV7 = 0x88898A8B,                    /* 環境設定7 レジスタ */
  RCUN1 = 0x8C8D8E8F,                    /* カウンタ1(指令位置) レジスタ */
  RCUN2 = 0x90919293,                    /* カウンタ2(汎用1) レジスタ */
  RCUN3 = 0x94959697,                    /* カウンタ3(偏差) レジスタ */
  RCUN4 = 0x98999A9B,                    /* カウンタ4(汎用2) レジスタ */
  RCMP1 = 0x9C9D9E9F,                    /* コンパレータ1比較値 レジスタ */
  RCMP2 = 0xA0A1A2A3,                    /* コンパレータ2比較値 レジスタ */
  RCMP3 = 0xA4A5A6A7,                    /* コンパレータ3比較値 レジスタ */
  RCMP4 = 0xA8A9AAAB,                    /* コンパレータ4比較値 レジスタ */
  RCMP5 = 0xACADAEAF,                    /* コンパレータ5比較値 レジスタ */
  RIRQ = 0xB0B1B2B3,                     /* イベント割り込み要求 レジスタ */
  RLTC1 = 0xB4B5B6B7,                    /* カウンタ1(指令位置)ラッチ レジスタ */
  RLTC2 = 0xB8B9BABB,                    /* カウンタ2(汎用1)ラッチ レジスタ */
  RLTC3 = 0xBCBDBEBF,                    /* カウンタ3(偏差)ラッチ レジスタ */
  RLTC4 = 0xC0C1C2C3,                    /* カウンタ4(汎用2)ラッチ レジスタ */
  RSTS = 0xC4C5C6C7,                     /* 拡張ステータス レジスタ */
  REST = 0xC8C9CACB,                     /* エラー割り込み要因 レジスタ */
  RIST = 0xCCCDCECF,                     /* イベント割り込み要因 レジスタ */
  RPLS = 0xD0D1D2D3,                     /* 残量パルス数 レジスタ */
  RSPD = 0xD4D5D6D7,                     /* 現在速度ステップ数 レジスタ */
  RSDC = 0xD8D9DADB,                     /* スローダウンポイント自動計算値 レジスタ */
  RCI = 0xF0F1F2F3,                      /* 円弧補間歩進数 レジスタ */
  RCIC = 0xF4F5F6F7,                     /* 円弧補間歩進カウンタ レジスタ */
  RIPS = 0xFCFDFEFF,                     /* 補間ステータス レジスタ */
};

/**
 * @brief PCL6046コマンド一覧
 * 
 */
enum Command{
  STAFL = 0x50,                          /* FL定速スタート */
  STAFH = 0x51,                          /* FH定速スタート */
  STAD = 0x52,                           /* 高速1スタート */
  STAUD = 0x53,                          /* 高速2スタート */
  CNTFL = 0x54,                          /* 残量FL定速スタート */
  CNTFH = 0x55,                          /* 残量FH定速スタート */
  CNTD = 0x56,                           /* 残量高速1スタート */
  CNTUD = 0x57,                          /* 残量高速2スタート */
  CMSTA = 0x06,                          /* 同時スタート(信号出力あり) */
  SPSTA = 0x2A,                          /* 同時スタート(信号出力なし) */
  FCHGL = 0x40,                          /* 瞬時にFL速度へ変更 */
  FCHGH = 0x41,                          /* 瞬時にFH速度へ変更 */
  FSCHL = 0x42,                          /* 減速してFL速度へ変更 */
  FSCHH = 0x43,                          /* 加速してFH速度へ変更 */
  STOP = 0x49,                           /* 即停止 */
  SDSTP = 0x4A,                          /* 減速停止 */
  CMSTP = 0x07,                          /* 同時停止(信号出力あり) */
  CMEMG = 0x05,                          /* 非常停止 */
  NOP = 0x00,                            /* NOPコマンド */
  P0RST = 0x10,                          /* 汎用出力ポートP0=L */
  P1RST = 0x11,                          /* 汎用出力ポートP1=L */
  P2RST = 0x12,                          /* 汎用出力ポートP2=L */
  P3RST = 0x13,                          /* 汎用出力ポートP3=L */
  P4RST = 0x14,                          /* 汎用出力ポートP4=L */
  P5RST = 0x15,                          /* 汎用出力ポートP5=L */
  P6RST = 0x16,                          /* 汎用出力ポートP6=L */
  P7RST = 0x17,                          /* 汎用出力ポートP7=L */
  P0SET = 0x18,                          /* 汎用出力ポートP0=H */
  P1SET = 0x19,                          /* 汎用出力ポートP1=H */
  P2SET = 0x1A,                          /* 汎用出力ポートP2=H */
  P3SET = 0x1B,                          /* 汎用出力ポートP3=H */
  P4SET = 0x1C,                          /* 汎用出力ポートP4=H */
  P5SET = 0x1D,                          /* 汎用出力ポートP5=H */
  P6SET = 0x1E,                          /* 汎用出力ポートP6=H */
  P7SET = 0x1F,                          /* 汎用出力ポートP7=H */
  SRST = 0x04,                           /* ソフトウェアリセット */
  CUN1R = 0x20,                          /* カウンタ1クリア */
  CUN2R = 0x21,                          /* カウンタ2クリア */
  CUN3R = 0x22,                          /* カウンタ3クリア */
  CUN4R = 0x23,                          /* カウンタ4クリア */
  ERCOUT = 0x24,                         /* ERC信号出力 */
  ERCRST = 0x25,                         /* ERC信号リセット */
  PRECAN = 0x26,                         /* プリレジスタキャンセル */
  PCPCAN = 0x27,                         /* コンパレータ5キャンセル */
  PRESHF = 0x2B,                         /* プリレジスタシフト */
  PCPSHF = 0x2C,                         /* コンパレータ5シフト */
  PRSET = 0x4F,                          /* プリレジスタオーバーライド確定 */
  STAON = 0x28,                          /* PCSコマンド */
  LTCH = 0x29,                           /* カウンタラッチ */
  SENIR = 0x2D,                          /* メインステータス動作割り込みステータスクリア */
  SEORR = 0x2E,                          /* メインステータスオーバーライドステータスクリア */
};

//---------------------------------------------------------------------------------------
//  型定義(Type definition)
//---------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------
//  プロトタイプ宣言(Prototype declaration)
//---------------------------------------------------------------------------------------

void arc_run(PIO pio, uint sm);         /* 円弧補間スタート */
void arc_init_pos(PIO pio, uint sm);    /* 円弧補間初期位置 */
void msts_chk(PIO pio, uint sm);        /* メインステータスBSY確認 */
void org_run(PIO pio, uint sm);         /* センサー原点位置移動 */
void pcl_reg_set(PIO pio, uint sm );    /* PCL初期設定レジスタ */

#endif  /* __MAIN_H__ */