/**
 * @file pcl_access.h
 * @author Nippon Pulse Motor Co., Ltd.
 * @brief PCL6046アクセス用(Z80フルアドレスでアクセス)
 * @version 0.1
 * @date 2023-06-07
 * 
 * @copyright Copyright (c) 2023
 * 
 */

#ifndef __PCL_ACCESS_H__
#define __PCL_ACCESS_H__

//---------------------------------------------------------------------------------------
//  型定義(Type definition)
//---------------------------------------------------------------------------------------
/**
 * @brief PCLアドレスデータ分割(ARM処理系)
 * 
 */
typedef union {
  uint32_t address;

  struct {
    uint8_t address_bytes_1;              /* PCLアドレスの最上位アドレス */
    uint8_t address_bytes_2;              /* PCLアドレスの上位アドレス */
    uint8_t address_bytes_3;              /* PCLアドレスの下位アドレス */
    uint8_t address_bytes_4;              /* PCLアドレスの最下位アドレス */
  }address_bytes;
} pcl_address;

/**
 * @brief PCL書き込みデータ分割(ARM処理系)
 * 
 */
typedef union {
  uint32_t data;
  
  struct {
    uint8_t data_bytes_1;                  /* PCLレジスタ書き込みの最下位データ */
    uint8_t data_bytes_2;                  /* PCLレジスタ書き込みの下位データ */
    uint8_t data_bytes_3;                  /* PCLレジスタ書き込みの上位データ */
    uint8_t data_bytes_4;                  /* PCLレジスタ書き込みの最上位データ */
  }data_bytes;
} write_data;

//---------------------------------------------------------------------------------------
//  プロトタイプ宣言(Prototype declaration)
//---------------------------------------------------------------------------------------

void pcl_bus_dir_change( PIO pio, uint sm, bool dir);                                                     /* PCLバスライン入出力方向切り替え */
void address_byte(uint32_t address, uint32_t pcl_address_bytes[]);                                        /* PCLアドレスデータ分割 */
void data_byte(uint32_t data, uint32_t reg_data_bytes[]);                                                 /* レジスタ書き込みデータ分割 */
void write_tx_fifo(PIO pio, uint sm, uint32_t write_data);                                                /* PIOステートマシンのTX_FIFOに書き込み */
uint32_t read_rx_fifo(PIO pio, uint sm, uint32_t write_data);                                             /* PIOステートマシンのRX_FIFOからデータ読み出し */

/* パラレルバス用ステートマシン初期化 */
void pcl_bus_init( PIO pio, uint sm, uint ifb, uint pio_out_base, uint pio_out_num, uint pio_side_base, uint pio_side_num, uint pio_in_base );
void write_reg_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address, uint32_t data );              /* PCLレジスタ書き込み関数 */
uint32_t read_reg_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address );                          /* PCLレジスタ読み出し関数 */
void write_com_pcl( PIO pio, uint sm, uint ifb, uint32_t com_axis, uint32_t address, uint32_t com );      /* PCLコマンド書き込み関数 */
uint32_t read_msts_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address );                         /* メインステータス読み出し関数 */
void read_port_substat_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address );                     /* 汎用ポート_サブステータス読み出し関数(未チェック) */
void write_port_pcl( PIO pio, uint sm, uint ifb, uint axs, uint32_t address, uint32_t data );             /* 汎用出力ポート書き込み関数(未チェック) */

#endif    /* __PCL_ACCESS_H__ */